@@ -17,9 +17,23 @@
 #
 
 function Usage() {
-    echo "Usage: $0 [platform] [keep]"
-    echo "Where \"platform\" is one of linux (default), ios, android, windows, freertos, openrtos-3.9.2, linux-ecc"
-    echo "Where \"keep\" means keep (default off) XXX-fips-test temp dir around for inspection"
+    printf '\n%s\n' "Usage: $0 [platform] [keep]"
+    printf '%s\n\n' "Where \"platform\" is one of:"
+    printf '\t%s\n' "linux (default)"
+    printf '\t%s\n' "ios"
+    printf '\t%s\n' "android"
+    printf '\t%s\n' "windows"
+    printf '\t%s\n' "freertos"
+    printf '\t%s\n' "openrtos-3.9.2"
+    printf '\t%s\n' "linux-ecc"
+    printf '\t%s\n' "netbsd-selftest"
+    printf '\t%s\n' "sgx"
+    printf '\t%s\n' "netos-7.6"
+    printf '\n%s\n\n' "Where \"keep\" means keep (default off) XXX-fips-test temp dir around for inspection"
+    printf '%s\n' "EXAMPLE:"
+    printf '%s\n' "---------------------------------"
+    printf '%s\n' "./fips-check.sh windows keep"
+    printf '%s\n\n' "---------------------------------"
 }
 
 LINUX_FIPS_VERSION=v3.2.6
@@ -57,11 +71,35 @@ OPENRTOS_3_9_2_FIPS_REPO=git@github.com:wolfSSL/fips.git
 OPENRTOS_3_9_2_CTAO_VERSION=v3.6.1
 OPENRTOS_3_9_2_CTAO_REPO=git@github.com:cyassl/cyassl.git
 
+#NOTE: Does not include the SGX examples yet, update version once fipsv2 is
+#      finished and merge conflicts can be resolved. This will be tagged as
+#      v3.12.4.sgx-examples
+#SGX_FIPS_VERSION=v3.12.4.sgx-examples
+SGX_FIPS_VERSION=v3.6.6
+SGX_FIPS_REPO=git@github.com:wolfSSL/fips.git
+SGX_CTAO_VERSION=v3.12.4
+SGX_CTAO_REPO=git@github.com:cyassl/cyassl.git
+
+NETOS_7_6_FIPS_VERSION=v3.12.6
+NETOS_7_6_FIPS_REPO=git@github.com:wolfSSL/fips.git
+NETOS_7_6_CTAO_VERSION=v3.12.4
+NETOS_7_6_CTAO_REPO=git@github.com:cyassl/cyassl.git
+
+
 FIPS_SRCS=( fips.c fips_test.c )
 WC_MODS=( aes des3 sha sha256 sha512 rsa hmac random )
 TEST_DIR=XXX-fips-test
 WC_INC_PATH=cyassl/ctaocrypt
 WC_SRC_PATH=ctaocrypt/src
+CAVP_SELFTEST_ONLY="no"
+
+# non-FIPS, CAVP only but pull in selftest
+# will reset above variables below in platform switch
+NETBSD_FIPS_VERSION=v3.14.2a
+NETBSD_FIPS_REPO=git@github.com:wolfssl/fips.git
+NETBSD_CTAO_VERSION=v3.14.2
+NETBSD_CTAO_REPO=git@github.com:wolfssl/wolfssl.git
+
 
 if [ "x$1" == "x" ]; then PLATFORM="linux"; else PLATFORM=$1; fi
 
@@ -111,6 +149,29 @@ linux-ecc)
   CTAO_VERSION=$LINUX_ECC_CTAO_VERSION
   CTAO_REPO=$LINUX_ECC_CTAO_REPO
   ;;
+netbsd-selftest)
+  FIPS_VERSION=$NETBSD_FIPS_VERSION
+  FIPS_REPO=$NETBSD_FIPS_REPO
+  CTAO_VERSION=$NETBSD_CTAO_VERSION
+  CTAO_REPO=$NETBSD_CTAO_REPO
+  FIPS_SRCS=( selftest.c )
+  WC_MODS=( dh ecc rsa dsa aes sha sha256 sha512 hmac random )
+  WC_INC_PATH=wolfssl/wolfcrypt
+  WC_SRC_PATH=wolfcrypt/src
+  CAVP_SELFTEST_ONLY="yes"
+  ;;
+sgx)
+  FIPS_VERSION=$SGX_FIPS_VERSION
+  FIPS_REPO=$SGX_FIPS_REPO
+  CTAO_VERSION=$SGX_CTAO_VERSION
+  CTAO_REPO=$SGX_CTAO_REPO
+  ;;
+netos-7.6)
+  FIPS_VERSION=$NETOS_7_6_FIPS_VERSION
+  FIPS_REPO=$NETOS_7_6_FIPS_REPO
+  CTAO_VERSION=$NETOS_7_6_CTAO_VERSION
+  CTAO_REPO=$NETOS_7_6_CTAO_REPO
+  ;;
 *)
   Usage
   exit 1
@@ -132,11 +193,17 @@ do
 done
 
 # The following is temporary. We are using random.c from a separate release
-pushd old-tree
-git checkout v3.6.0
-popd
-cp old-tree/$WC_SRC_PATH/random.c $WC_SRC_PATH
-cp old-tree/$WC_INC_PATH/random.h $WC_INC_PATH
+# This is forcefully overwriting any other checkout of the cyassl sources.
+# Removing this as default behavior for SGX and netos projects.
+if [ "x$CAVP_SELFTEST_ONLY" == "xno" ] && [ "x$PLATFORM" != "xsgx" ] && \
+   [ "x$PLATFORM" != "xnetos-7.6" ];
+then
+    pushd old-tree
+    git checkout v3.6.0
+    popd
+    cp old-tree/$WC_SRC_PATH/random.c $WC_SRC_PATH
+    cp old-tree/$WC_INC_PATH/random.h $WC_INC_PATH
+fi
 
 # clone the FIPS repository
 git clone -b $FIPS_VERSION $FIPS_REPO fips
@@ -149,14 +216,22 @@ done
 
 # run the make test
 ./autogen.sh
-./configure --enable-fips
+if [ "x$CAVP_SELFTEST_ONLY" == "xyes" ];
+then
+    ./configure --enable-selftest
+else
+    ./configure --enable-fips
+fi
 make
 [ $? -ne 0 ] && echo "\n\nMake failed. Debris left for analysis." && exit 1
 
-NEWHASH=`./wolfcrypt/test/testwolfcrypt | sed -n 's/hash = \(.*\)/\1/p'`
-if [ -n "$NEWHASH" ]; then
-    sed -i.bak "s/^\".*\";/\"${NEWHASH}\";/" $WC_SRC_PATH/fips_test.c
-    make clean
+if [ "x$CAVP_SELFTEST_ONLY" == "xno" ];
+then
+    NEWHASH=`./wolfcrypt/test/testwolfcrypt | sed -n 's/hash = \(.*\)/\1/p'`
+    if [ -n "$NEWHASH" ]; then
+        sed -i.bak "s/^\".*\";/\"${NEWHASH}\";/" $WC_SRC_PATH/fips_test.c
+        make clean
+    fi
 fi
 
 make test
