@@ -56,6 +56,7 @@ static char flagSep[] = " ";
     static char svrPort[] = "0";
 #endif
 static char forceDefCipherListFlag[] = "-HdefCipherList";
+static char exitWithRetFlag[] = "-HexitWithRet";
 
 #ifdef WOLFSSL_ASYNC_CRYPT
     static int devId = INVALID_DEVID;
@@ -161,7 +162,7 @@ static int execute_test_case(int svr_argc, char** svr_argv,
                               int cli_argc, char** cli_argv,
                               int addNoVerify, int addNonBlocking,
                               int addDisableEMS, int forceSrvDefCipherList,
-                              int forceCliDefCipherList)
+                              int forceCliDefCipherList, int testShouldFail)
 {
 #ifdef WOLFSSL_TIRTOS
     func_args cliArgs = {0};
@@ -231,7 +232,7 @@ static int execute_test_case(int svr_argc, char** svr_argv,
     }
 #endif
 
-    /* Build Client Command */
+    /* Build Server Command */
     if (addNoVerify) {
         printf("repeating test with client cert request off\n");
         if (svrArgs.argc >= MAX_ARGS)
@@ -261,6 +262,12 @@ static int execute_test_case(int svr_argc, char** svr_argv,
         else
             svr_argv[svrArgs.argc++] = forceDefCipherListFlag;
     }
+#ifdef TEST_PK_PRIVKEY
+    svr_argv[svrArgs.argc++] = (char*)"-P";
+#endif
+    if (testShouldFail) {
+        svr_argv[svrArgs.argc++] = exitWithRetFlag;
+    }
 
     /* update server flags list */
     commandLine[0] = '\0';
@@ -321,6 +328,12 @@ static int execute_test_case(int svr_argc, char** svr_argv,
         else
             cli_argv[cliArgs.argc++] = forceDefCipherListFlag;
     }
+#ifdef TEST_PK_PRIVKEY
+    cli_argv[cliArgs.argc++] = (char*)"-P";
+#endif
+    if (testShouldFail) {
+        cli_argv[cliArgs.argc++] = exitWithRetFlag;
+    }
 
     commandLine[0] = '\0';
     added = 0;
@@ -339,13 +352,15 @@ static int execute_test_case(int svr_argc, char** svr_argv,
     client_test(&cliArgs);
 
     /* verify results */
-    if (cliArgs.return_code != 0) {
+    if ((cliArgs.return_code != 0 && testShouldFail == 0) ||
+        (cliArgs.return_code == 0 && testShouldFail != 0)) {
         printf("client_test failed\n");
         exit(EXIT_FAILURE);
     }
 
     join_thread(serverThread);
-    if (svrArgs.return_code != 0) {
+    if ((svrArgs.return_code != 0 && testShouldFail == 0) ||
+        (svrArgs.return_code == 0 && testShouldFail != 0)) {
         printf("server_test failed\n");
         exit(EXIT_FAILURE);
     }
@@ -355,6 +370,11 @@ static int execute_test_case(int svr_argc, char** svr_argv,
 #endif
     FreeTcpReady(&ready);
 
+    /* only run the first test for failure cases */
+    if (testShouldFail) {
+        return NOT_BUILT_IN;
+    }
+
     return 0;
 }
 
@@ -373,18 +393,23 @@ static void test_harness(void* vargs)
     char* cursor;
     char* comment;
     const char* fname = "tests/test.conf";
+    int   testShouldFail = 0;
 
     if (args->argc == 1) {
         printf("notice: using default file %s\n", fname);
     }
-    else if(args->argc != 2) {
-        printf("usage: harness [FILE]\n");
+    else if(args->argc > 3) {
+        printf("usage: harness [FILE] [ARG]\n");
         args->return_code = 1;
         return;
     }
-    else {
+
+    if (args->argc >= 2) {
         fname = args->argv[1];
     }
+    if (args->argc == 3) {
+        testShouldFail = 1;
+    }
 
     file = fopen(fname, "rb");
     if (file == NULL) {
@@ -443,7 +468,7 @@ static void test_harness(void* vargs)
                 break;
             case '#':
                 /* Ignore lines that start with a #. */
-                comment = strsep(&cursor, "\n");
+                comment = XSTRSEP(&cursor, "\n");
 #ifdef DEBUG_SUITE_TESTS
                 printf("%s\n", comment);
 #else
@@ -454,20 +479,23 @@ static void test_harness(void* vargs)
                 /* Parameters start with a -. They end in either a newline
                  * or a space. Capture until either, save in Args list. */
                 if (cliMode)
-                    cliArgs[cliArgsSz++] = strsep(&cursor, " \n");
+                    cliArgs[cliArgsSz++] = XSTRSEP(&cursor, " \n");
                 else
-                    svrArgs[svrArgsSz++] = strsep(&cursor, " \n");
+                    svrArgs[svrArgsSz++] = XSTRSEP(&cursor, " \n");
+                if (*cursor == 0)  /* eof */
+                    do_it = 1;
                 break;
             default:
                 /* Anything from cursor until end of line that isn't the above
                  * is data for a paramter. Just up until the next newline in
                  * the Args list. */
                 if (cliMode)
-                    cliArgs[cliArgsSz++] = strsep(&cursor, "\n");
+                    cliArgs[cliArgsSz++] = XSTRSEP(&cursor, "\n");
                 else
-                    svrArgs[svrArgsSz++] = strsep(&cursor, "\n");
+                    svrArgs[svrArgsSz++] = XSTRSEP(&cursor, "\n");
                 if (*cursor == 0)  /* eof */
                     do_it = 1;
+                break;
         }
 
         if (svrArgsSz == MAX_ARGS || cliArgsSz == MAX_ARGS) {
@@ -477,31 +505,41 @@ static void test_harness(void* vargs)
 
         if (do_it) {
             ret = execute_test_case(svrArgsSz, svrArgs,
-                                    cliArgsSz, cliArgs, 0, 0, 0, 0, 0);
+                                    cliArgsSz, cliArgs, 0, 0, 0, 0, 0,
+                                    testShouldFail);
             /* don't repeat if not supported in build */
             if (ret == 0) {
                 /* test with default cipher list on server side */
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 0, 0, 0, 1, 0);
+                                  cliArgsSz, cliArgs, 0, 0, 0, 1, 0,
+                                  testShouldFail);
                 /* test with default cipher list on client side */
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 0, 0, 0, 0, 1);
+                                  cliArgsSz, cliArgs, 0, 0, 0, 0, 1,
+                                  testShouldFail);
 
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 0, 1, 0, 0, 0);
+                                  cliArgsSz, cliArgs, 0, 1, 0, 0, 0,
+                                  testShouldFail);
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 1, 0, 0, 0, 0);
+                                  cliArgsSz, cliArgs, 1, 0, 0, 0, 0,
+                                  testShouldFail);
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 1, 1, 0, 0, 0);
+                                  cliArgsSz, cliArgs, 1, 1, 0, 0, 0,
+                                  testShouldFail);
 #ifdef HAVE_EXTENDED_MASTER
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 0, 0, 1, 0, 0);
+                                  cliArgsSz, cliArgs, 0, 0, 1, 0, 0,
+                                  testShouldFail);
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 0, 1, 1, 0, 0);
+                                  cliArgsSz, cliArgs, 0, 1, 1, 0, 0,
+                                  testShouldFail);
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 1, 0, 1, 0, 0);
+                                  cliArgsSz, cliArgs, 1, 0, 1, 0, 0,
+                                  testShouldFail);
                 execute_test_case(svrArgsSz, svrArgs,
-                                  cliArgsSz, cliArgs, 1, 1, 1, 0, 0);
+                                  cliArgsSz, cliArgs, 1, 1, 1, 0, 0,
+                                  testShouldFail);
 #endif
             }
             svrArgsSz = 1;
@@ -520,14 +558,15 @@ int SuiteTest(void)
 {
 #if !defined(NO_WOLFSSL_SERVER) && !defined(NO_WOLFSSL_CLIENT)
     func_args args;
-    char argv0[2][80];
-    char* myArgv[2];
+    char argv0[3][80];
+    char* myArgv[3];
 
     printf(" Begin Cipher Suite Tests\n");
 
     /* setup */
     myArgv[0] = argv0[0];
     myArgv[1] = argv0[1];
+    myArgv[2] = argv0[2];
     args.argv = myArgv;
     strcpy(argv0[0], "SuiteTest");
 
@@ -537,7 +576,7 @@ int SuiteTest(void)
 
     (void)test_harness;
 
-    cipherSuiteCtx = wolfSSL_CTX_new(wolfTLSv1_2_client_method());
+    cipherSuiteCtx = wolfSSL_CTX_new(wolfSSLv23_client_method());
     if (cipherSuiteCtx == NULL) {
         printf("can't get cipher suite ctx\n");
         exit(EXIT_FAILURE);
@@ -595,6 +634,16 @@ int SuiteTest(void)
         exit(EXIT_FAILURE);
     }
     #endif
+    #ifndef WOLFSSL_NO_TLS12
+    /* add TLSv13 downgrade tets */
+    strcpy(argv0[1], "tests/test-tls13-down.conf");
+    printf("starting TLSv13 Downgrade extra tests\n");
+    test_harness(&args);
+    if (args.return_code != 0) {
+        printf("error from script %d\n", args.return_code);
+        exit(EXIT_FAILURE);
+    }
+    #endif
 #endif
 #if defined(HAVE_CURVE25519) && defined(HAVE_ED25519)
     /* add ED25519 certificate cipher suite tests */
@@ -652,11 +701,36 @@ int SuiteTest(void)
         goto exit;
     }
 #endif
-
 #ifndef NO_PSK
+    #ifndef WOLFSSL_NO_TLS12
+        #if !defined(NO_RSA) || defined(HAVE_ECC)
+        /* add psk cipher suites */
+        strcpy(argv0[1], "tests/test-psk.conf");
+        printf("starting psk cipher suite tests\n");
+        test_harness(&args);
+        if (args.return_code != 0) {
+            printf("error from script %d\n", args.return_code);
+            args.return_code = EXIT_FAILURE;
+            goto exit;
+        }
+        #endif
+    #endif
+    #ifdef WOLFSSL_TLS13
     /* add psk extra suites */
-    strcpy(argv0[1], "tests/test-psk-no-id.conf");
-    printf("starting psk no identity extra cipher suite tests\n");
+    strcpy(argv0[1], "tests/test-tls13-psk.conf");
+    printf("starting TLS 1.3 psk no identity extra cipher suite tests\n");
+    test_harness(&args);
+    if (args.return_code != 0) {
+        printf("error from script %d\n", args.return_code);
+        args.return_code = EXIT_FAILURE;
+        goto exit;
+    }
+    #endif
+#endif
+#if defined(WOLFSSL_ENCRYPTED_KEYS) && !defined(NO_DES3)
+    /* test encrypted keys */
+    strcpy(argv0[1], "tests/test-enckeys.conf");
+    printf("starting encrypted keys extra cipher suite tests\n");
     test_harness(&args);
     if (args.return_code != 0) {
         printf("error from script %d\n", args.return_code);
@@ -665,6 +739,18 @@ int SuiteTest(void)
     }
 #endif
 
+    /* failure tests */
+    args.argc = 3;
+    strcpy(argv0[1], "tests/test-fails.conf");
+    strcpy(argv0[2], "-f");
+    printf("starting tests that expect failure\n");
+    test_harness(&args);
+    if (args.return_code != 0) {
+        printf("error from script %d\n", args.return_code);
+        args.return_code = EXIT_FAILURE;
+        goto exit;
+    }
+
 exit:
     printf(" End Cipher Suite Tests\n");
 
